#include "mtdplot.h"
#include "mobjectx.h"
#include "intervalle.h"
#include "syscoord.h"

/////////////////////////////////////////////////////////////////////////////////////////////////////////
int GetDimFor(TMethod plt)
{
	if (plt == TM_UNKNOWN) return 0;
	return (plt <= TM_POLAR)? 2:3;
}

TMethod GetTypeMethodFor(char *meth)
{
	for (int i=0; XM_KEYWORD[i]; i++) if (!strcmp(meth,XM_KEYWORD[i])) return ((TMethod)i);
	return TM_UNKNOWN;
}

XMMethodPlot *GetMethodFor(TMethod met)
{
	switch (met)
	{
		case TM_CARTESIAN2D:	return (new XMMethodCar2D);
		case TM_PARAMETRIC2D:	return (new XMMethodCar2D);
		case TM_POLAR:		return (new XMMethodPol);
		case TM_CARTESIAN3D:	return (new XMMethodCar3D);
		case TM_PARAMETRIC3D: 	return (new XMMethodCar3D);
		case TM_CYLINDRIC: 	return (new XMMethodCyl);
		case TM_SPHERIC:	return (new XMMethodSph);
		default: return 0;
	}
}
/////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////////////////////////////////////////
XMMap *XMMethodCar2D::Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys)
{
	if (!sys) return NULL;
	CList<XMIntervalle *> *asys = (CList<XMIntervalle *> *)(sys);
	TYPE Res[3];
	int vld;int k=0;
	XMMap *map = new XMMap(2);
	for (asys->Get(0)->Init();asys->Get(0)->IsValid();asys->Get(0)->Inc())
	{
		Res[2] = (float)(lob->Get(0)->Eval((int &)(Res[0]),sys));
		Res[1] = (float)(asys->Get(0)->Eval(vld));
		map->Add(Res);
	}
	printf("<<<<<<<<<<  Nombre d'elements: %d   >>>>>>>>>>>>>>\n", map->GetCount());
	return map;
}

void *XMMethodCar2D::DefaultAxis()
{
	SysCoord *sys;
	sys = new SysCoord(2);
	sys->ConfigureAxe(0,"x");
	sys->ConfigureAxe(1,"y");
	return ((void *)sys);
}

TYPE *XMMethodCar2D::Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res)
{
	int v[2];
	Res[0] = pobj[0]->Eval(v[0]);
	Res[1] = pobj[1]->Eval(v[1]);
	vld = v[0] && v[1];
	return Res;
}

TYPE XMMethodCar2D::EvalX(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return pobj[0]->Eval(vld);
}

TYPE XMMethodCar2D::EvalY(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return pobj[1]->Eval(vld);
}

TYPE XMMethodCar2D::EvalZ(int &vld, XMObjectX **pobj, TYPE *pt)
{
	vld = 1;
	return 0;
}

TYPE *XMMethodPol::Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res)
{
	TYPE rho;
	rho = pobj[0]->Eval(vld);		//rho(theta)
	Res[0] = rho*cos(pt[0]);
	Res[1] = rho* sin(pt[0]);
	return Res;
}

TYPE XMMethodPol::EvalX(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return (pobj[0]->Eval(vld))* cos(pt[0]);
}

TYPE XMMethodPol::EvalY(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return (pobj[1]->Eval(vld))* sin(pt[0]);
}

TYPE XMMethodPol::EvalZ(int &vld, XMObjectX **pobj, TYPE *pt)
{
	vld = 1;
	return 0;
}

TYPE *XMMethodCar3D::Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res)
{
	int v[3];
	Res[0] = pobj[0]->Eval(v[0]);
	Res[1] = pobj[1]->Eval(v[1]);
	Res[2] = pobj[2]->Eval(v[2]);
	vld = v[0] && v[1]&& v[2];
	return Res;
}

TYPE XMMethodCar3D::EvalZ(int &vld, XMObjectX **pobj, TYPE *pt)
{
	int v;
	return pobj[2]->Eval(v);
}

TYPE *XMMethodCyl::Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res)
{
	int v[2];
	TYPE rho;
	rho = pobj[0]->Eval(v[0]);	// rho(theta);
	Res[0] = rho*cos(pt[0]);
	Res[1] = rho* sin(pt[0]);
	Res[2] = pobj[1]->Eval(v[1]);
	vld = v[0] && v[1];
	return Res;
}

TYPE XMMethodCyl::EvalZ(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return  pobj[1]->Eval(vld);
}

TYPE *XMMethodSph::Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res)
{
	TYPE r;
	r = pobj[0]->Eval(vld);				// r(phi, theta);
	Res[0] = r*sin(pt[0])*cos(pt[1]);		//
	Res[1] = r*sin(pt[0])*sin(pt[1]);
	Res[2] = r*cos(pt[0]);
	return Res;
}

TYPE XMMethodSph::EvalX(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return (pobj[0]->Eval(vld))*sin(pt[0])*cos(pt[1]);
}

TYPE XMMethodSph::EvalY(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return (pobj[0]->Eval(vld))*sin(pt[0])*sin(pt[1]);
}

TYPE XMMethodSph::EvalZ(int &vld, XMObjectX **pobj, TYPE *pt)
{
	return (pobj[0]->Eval(vld))*cos(pt[0]);
}