#ifndef MTDPLOT_H
#define MTDPLOT_H
#include "type.h"
#include "map.h"

/*
*	Class de base des methodes de tracer
*/
/////////////////////////////
int GetDimFor(TMethod plt);//
/////////////////////////////

/////////////////////////////////////////////////////////////
//  Classe de base pour les diffrentes mthodes de tracer //
/////////////////////////////////////////////////////////////
class XMMethodPlot
{
protected:
	TMethod mTMethod;
	int mDim;
public:
	XMMethodPlot(){mTMethod = TM_UNKNOWN;}
	XMMethodPlot(TMethod methode){mTMethod = methode;mDim = GetDimFor(methode);}
	TMethod &Method(){return mTMethod;}
	int &Dim(){return mDim;}
	virtual void *DefaultAxis()=0;
	virtual XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys)=0;
	virtual TYPE *Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res)=0;
	virtual TYPE EvalX(int &vld, XMObjectX **pobj, TYPE *pt)=0;
	virtual TYPE EvalY(int &vld, XMObjectX **pobj, TYPE *pt)=0;
	virtual TYPE EvalZ(int &vld, XMObjectX **pobj, TYPE *pt)=0;
};
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
//	Trace en cartesienne 2D				 //
///////////////////////////////////////////////////////////////////
class XMMethodCar2D: public XMMethodPlot
{
protected:
	XMMethodCar2D(TMethod meth):XMMethodPlot(meth){};
public:
	XMMethodCar2D(): XMMethodPlot(TM_CARTESIAN2D){};
	void *DefaultAxis();
	XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys);
	TYPE *Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res);
	TYPE EvalX(int &vld, XMObjectX **pobj, TYPE *pt);
	TYPE EvalY(int &vld, XMObjectX **pobj, TYPE *pt);
	TYPE EvalZ(int &vld, XMObjectX **pobj, TYPE *pt);
};
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
//	NI* Trace en Parametrique 2D				 //
///////////////////////////////////////////////////////////////////
class XMMethodPar2D: public XMMethodCar2D
{
protected:
	XMMethodPar2D(TMethod meth):XMMethodCar2D(meth){};
public:
	XMMethodPar2D():XMMethodCar2D(TM_PARAMETRIC2D){};
};
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
//	Trace en Polaire					 //
///////////////////////////////////////////////////////////////////
class XMMethodPol: public XMMethodPlot
{
protected:
	XMMethodPol(TMethod meth):XMMethodPlot(meth){};
public:
	XMMethodPol(): XMMethodPlot(TM_POLAR){};
	void *DefaultAxis(){};
	XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys){};
	TYPE *Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res);
	TYPE EvalX(int &vld, XMObjectX **pobj, TYPE *pt);
	TYPE EvalY(int &vld, XMObjectX **pobj, TYPE *pt);
	TYPE EvalZ(int &vld, XMObjectX **pobj, TYPE *pt);
};
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
//	Trace en cartsienne 3D				 //
///////////////////////////////////////////////////////////////////
class XMMethodCar3D: public XMMethodCar2D
{
protected:
	XMMethodCar3D(TMethod meth):XMMethodCar2D(meth){};
public:
	XMMethodCar3D():XMMethodCar2D(TM_CARTESIAN3D){};
	XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys){};
	TYPE *Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res);
	TYPE EvalZ(int &vld, XMObjectX **pobj, TYPE *pt);
};
///////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////
//	NI* Trace en Paramtrique 3D				 //
///////////////////////////////////////////////////////////////////
class XMMethodPar3D: public XMMethodPar2D
{
protected:
	XMMethodPar3D(TMethod meth):XMMethodPar2D(meth){};
public:
	XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys){};
	XMMethodPar3D():XMMethodPar2D(TM_PARAMETRIC2D){};
};
///////////////////////////////////////////////////////////////////

///////////////////////////////////////////////////////////////////
//	Trace en cylindrique					 //
///////////////////////////////////////////////////////////////////
class XMMethodCyl: public XMMethodPol
{
	XMMethodCyl(TMethod meth): XMMethodPol(meth){};
public:
	XMMethodCyl(): XMMethodPol(TM_CYLINDRIC){};
	XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys){};
	TYPE *Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res);
	TYPE EvalZ(int &vld, XMObjectX **pobj, TYPE *pt);
};
///////////////////////////////////////////////////////////////////


///////////////////////////////////////////////////////////////////
//	NI* Trace en sphrique					 //
///////////////////////////////////////////////////////////////////
class XMMethodSph: public XMMethodPlot
{
public:
	XMMethodSph(): XMMethodPlot(TM_SPHERIC){};
	void *DefaultAxis(){};
	XMMap *Map(CList <XMObjectX *> *lob, CList<XMObjectX *> *sys){};
	TYPE *Reach(int &vld, XMObjectX **pobj, TYPE *pt, TYPE *Res);
	TYPE EvalX(int &vld, XMObjectX **pobj, TYPE *pt);
	TYPE EvalY(int &vld, XMObjectX **pobj, TYPE *pt);
	TYPE EvalZ(int &vld, XMObjectX **pobj, TYPE *pt);
};
///////////////////////////////////////////////////////////////////

//////////////////////////////////////////
XMMethodPlot *GetMethodFor(TMethod met);//
TMethod GetTypeMethodFor(char *meth);	//
//////////////////////////////////////////
#endif	/* MTDPLOT_H */